import numpy as np
from matplotlib.textpath import TextPath
from matplotlib.patches import PathPatch, FancyBboxPatch
from pyquaternion import Quaternion
from os.path import exists

def StrToBool(s):
	if isinstance(s, bool):
		return s
	if s.lower() in ('yes', 'true', 't', 'y', '1'):
		return True
	elif s.lower() in ('no', 'false', 'f', 'n', '0'):
		return False
	else:
		raise argparse.ArgumentTypeError('Boolean value expected.')

def StrToNumpy(s):
	if not s:
		return np.array([])
	else:
		l = list(map(int,s.split(':')))
		return np.array(l) if len(l)==1 else  np.arange(l[0],l[1]) if len(l)==2 else np.arange(l[0],l[1],l[2])

def Arrow(ax,Xa,Xb,**opt):
	def get_key(key,default_value):
		return default_value if key not in opt else opt[key]
	arrowstyle = get_key('arrowstyle','-')
	xycoords = get_key('xycoords','data')
	if (any(elem in arrowstyle for elem in '<>')):
		head_length = get_key('head_length','0.8');
		head_width  = get_key('head_width' ,'0.4');
		arrowstyle+=',head_length = ' + str(head_length) + ',head_width = ' + str(head_width)
	return ax.annotate(get_key('text',''),
	                   xytext = (Xa[0],Xa[1]),
	                   xy = (Xb[0],Xb[1]),
	                   xycoords = xycoords,
	                   textcoords = xycoords,
	                   zorder = get_key('zorder',-1),
	                   annotation_clip = get_key('annotation_clip',False),
	                   ha='center',
	                   va='center',
	                   arrowprops = dict(arrowstyle = arrowstyle,
	                                     shrinkA = get_key('shrinkA',0),
	                                     shrinkB = get_key('shrinkB',0),
	                                     linestyle = get_key('linestyle','-'),
	                                     linewidth = get_key('linewidth',1),
	                                     facecolor = get_key('color','black'),
	                                     edgecolor = get_key('color','black'),
	                                     clip_path = get_key('clip',None)))

def TransformFigToAx(ax,x_fig,y_fig):
	bbox = ax.get_position()
	xlim = ax.get_xlim()
	ylim = ax.get_ylim()
	alphax = (x_fig-bbox.x0) / (bbox.x1-bbox.x0)
	alphay = (y_fig-bbox.y0) / (bbox.y1-bbox.y0)
	x_ax = xlim[0] + alphax*(xlim[1]-xlim[0]);
	y_ax = ylim[0] + alphay*(ylim[1]-ylim[0]);
	return x_ax,y_ax

def TransformAxToFig(ax,x_ax,y_ax):
	bbox = ax.get_position()
	xlim = ax.get_xlim()
	ylim = ax.get_ylim()
	alphax = (x_ax-xlim[0])/(xlim[1]-xlim[0])
	alphay = (y_ax-ylim[0])/(ylim[1]-ylim[0])
	xnew = bbox.x0 + alphax * (bbox.x1-bbox.x0)
	ynew = bbox.y0 + alphay * (bbox.y1-bbox.y0)
	return xnew,ynew

def TextBox(ax,X,text,**opt):
	def get_key(key,default_value):
		return default_value if key not in opt else opt[key]
	tp = TextPath((0,0),
	              text,
	              size = get_key('size',10),
	              usetex = get_key('usetex',True))
	bb = tp.get_extents()
	xm,ym = (bb.x0 + bb.x1)/2,(bb.y0 + bb.y1)/2
	tp.vertices[:,0] = np.add(tp.vertices[:,0],-xm+X[0])
	tp.vertices[:,1] = np.add(tp.vertices[:,1],-ym+X[1])
	pp = PathPatch(tp,
	               facecolor = get_key('textcolor','black'),
	               edgecolor = 'None',
	               #linewidth = None,
	               clip_on = get_key('clip_on',False),
	               zorder = get_key('zorder',-1))
	bb = tp.get_extents()
	rect = FancyBboxPatch((bb.x0, bb.y0),
	                      bb.width,
	                      bb.height,
	                      boxstyle = 'round,pad={0:},rounding_size={1:}'.format(get_key('pad',0),get_key('rounding_size',0)),
	                      facecolor = get_key('facecolor','white'),
	                      edgecolor = get_key('edgecolor','black'),
	                      linewidth = get_key('linewidth',1),
	                      clip_on = get_key('clip_on',False),
	                      alpha = get_key('alpha_box',1),
	                      zorder = get_key('zorder',-1))
	ax.add_artist(rect)
	ax.add_artist(pp)

#
#  PETSC POSTPRCESSING STUFF
#

def GetFileName(name,time=-1):
	if name[-4:] == ".txt":
		return name
	else:
		tExtension = ''
		if time>-1:
			tExtension = '_t{t:05d}'.format(t=time)
		return name + tExtension + '.txt'

def GetPolylineName(name,pl=-1,time=-1):
	if name[-4:] == ".txt":
		return name
	else:
		pExtension = ''
		tExtension = ''
		if pl>-1:
			pExtension = '{p:02d}'.format(p=pl)
		if time>-1:
			tExtension = '_t{t:05d}'.format(t=time)
		return name + pExtension + tExtension + '.txt'

def ImportPolyline(my_path='.',name='tpcl.txt',skiprows=0,info=True):
	my_file = my_path + '/' + name
	xs,ys,zs = np.loadtxt(my_file,usecols=(0,1,2),skiprows=skiprows,unpack=True)
	if (xs.size == 0):
		print("Error: empty file")
		sys.exit()
	if info:
		print("  Imported {} points from {}".format(xs.size,name))
	return np.transpose([xs,ys,zs])

def GetPolylineLength(polyline):
	if polyline.size > 0:
		diff = np.diff(polyline,axis=0)
		norm = np.linalg.norm(diff,axis=1)
		return np.sum(norm)
	return 0

def ImportPolylines(my_path='.',name='tpcl',cloudsize=-1,time=-1,smooth=-1,info=True):
	fs = []
	polylines = []
	if cloudsize > 0:
		#cloud
		for pl in range(cloudsize):
			f = GetPolylineName(name=name,pl=pl,time=time)
			fs.append(f)
	else:
		#single polyline
		f = GetPolylineName(name=name,time=time)
		fs.append(f)
	for f in fs:
		if exists(my_path + '/' + f):
			polyline = ImportPolyline(my_path=my_path,name=f,info=info)
			if (smooth > 0):
				polyline = SmoothPolyline(polyline,smooth=smooth,info=True)
			polylines.append(polyline)
		else:
			polylines.append(np.array([]))
	return polylines

def GetCapillaryForce(my_path='.',name='tpcl',theta=90.,time=-1,info=True):
	sinTheta = np.sin(np.pi*theta/180)
	def GetPivot(my_path,time):
		side_name = GetFileName(my_path + "/side0",time)
		data = np.loadtxt(side_name)
		m = data[0:9].reshape(3,3,order='F')
		A,B,C = m[0],m[1],m[2]
		#h = data[9]
		n = data[10:13]
		centre = data[13:16]
		q = data[16:20]
		q = Quaternion(np.roll(q,1))
		R = q.rotation_matrix
		return centre + R.dot(A+B)/2
	def GetN(my_path,time):
		side_name = GetFileName(my_path + "/side0",time)
		data = np.loadtxt(side_name)
		n = data[10:13]
		q = data[16:20]
		q = Quaternion(np.roll(q,1))
		R = q.rotation_matrix
		return R.dot(n)
	def GetForce(polylines,pivot,n):
		forces,arms = [],[]
		for polyline in polylines:
			length = GetPolylineLength(polyline)
			if length>0:
				force = length * n
				arm = np.mean(polyline,axis=0) - pivot
				forces.append(force)
			else:
				forces.append(np.zeros(3))
				arms.append(np.zeros(3))
			#if polyline.size > 0:
				#diff = np.diff(polyline,axis=0)
				#norm = np.linalg.norm(diff,axis=1)
				#length = np.sum(norm)
				#length = GetPolylineLength(polyline)
				#force = length * n
				#arm = np.mean(polyline,axis=0) - pivot
				#forces.append(force)
				#arms.append(arm)
			#else:
				#forces.append(np.zeros(3))
				#arms.append(np.zeros(3))
		forces = np.array(forces) * sinTheta
		torques = np.array([np.cross(arm,force) for arm,force in zip(arms,forces)])
		return np.sum(forces,axis=0),np.sum(torques,axis=0)
	polylines = ImportPolylines(my_path = my_path,
	                            name = name,
	                            cloudsize = 10, #extra large
	                            time = time,
	                            smooth = -1,
	                            info = info)
	pivot = GetPivot(my_path,time)
	n = GetN(my_path,time)
	Fc,Tc = GetForce(polylines,pivot,n)
	return Fc,Tc

def GetStructuralReferenceForce(my_path='.',name='hinge0',time=0):
	hinge_name = GetFileName(my_path + '/hinge0',time)
	data = np.loadtxt(hinge_name).transpose()
	radii = data[1]
	radius = radii[0]
	n = radii.size
	ks = 1
	length = 2*(n-1)*radius
	return 0.5 * ks * (radius**3) / (length**2)

def GetNormalizedCapillaryForce(my_path='.',name='tpcl',theta=90,ec=1,time=-1,info=True):
	if info:
		print('  my_path = {}'.format(my_path))
		print('  theta = {}'.format(theta))
		print('  ec = {}'.format(ec))
	F,T = GetCapillaryForce(my_path=my_path,name=name,time=time,theta=theta,info=info)
	F_ref = GetStructuralReferenceForce(my_path=my_path,name='hinge0',time=time)
	vb = 3800
	L3_ref = vb/(2*np.pi/3)
	L1_ref = L3_ref**(1/3)
	L2_ref = L3_ref**(2/3)
	F = ec*F*F_ref/L1_ref
	T = ec*T*F_ref/L2_ref
	return F,T
