#!/usr/bin/python3
import numpy as np
from Utils import *
import argparse
from pyquaternion import Quaternion
import matplotlib.pyplot as plt
from matplotlib.patches import Circle
from matplotlib.collections import PatchCollection
import matplotlib
matplotlib.rc('text', usetex=True)
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{bm}"]
matplotlib.rc('text', usetex=True)

def GetTopSide0(my_path='.',ca=90,side=3,ec=1.0,time=0,transform=True):
	name = GetFileName(my_path + '/ca{}/side{}/ec{:.1f}/side0'.format(ca,side,ec),time=time)
	data = np.loadtxt(name)
	m = data[0:9].reshape(3,3,order='F')
	#A,B,C = m[0],m[1],m[2]
	h = data[9]
	n = data[10:13]
	c = data[13:16]
	q = data[16:20]
	q = Quaternion(np.roll(q,1)).normalised
	m = m + h*n/2
	if transform:
		R = q.rotation_matrix
		n = R.dot(n)
		m = np.array([c + R.dot(row) for row in m])
	return m,[h,n,c,q]

#GetAlphas(my_path='',iterations):
	#alphas = np.empty(iterations.size)
	#for iteration in interations:
		#alpha[iteration] = 0
	#return alpha

def GetForce(my_path='.',ca=90,side=3,ec=1.0,every=50):
	my_newpath = my_path + '/ca{}/side{}/ec{:.1f}'.format(ca,side,ec)
	force_ref = GetStructuralReferenceForce(my_path=my_newpath,name='hinge0',time=0)
	data = np.loadtxt(my_newpath + '/force.txt',skiprows=1)
	iterations = data[:,0]
	#qs = [Quaternion(qw,qx,qy,qz).normalised for (qx,qy,qz,qw) in data[:,[1,2,3,4]]]
	forces = data[:,[5,6,7]]
	forces_mag = np.linalg.norm(forces,axis=1)
	times = iterations[::every]/every
	#xs = [q.degrees for q in qs[::every]]
	forces = forces_mag[::every]/force_ref
	return times,forces

#~~~~~~~~~~
#  Parser
#~~~~~~~~~~
parser = argparse.ArgumentParser()
parser.add_argument('-ca',type=int,default=90,help='contact angle')
parser.add_argument('-sides',type=StrToNumpy,default="3:8",help='number of sides')
parser.add_argument('-ec',type=float,default=4.0,help='Ec')
parser.add_argument('-every',type=int,default=50,help='every')
args = parser.parse_args()

#~~~~~~
#  Ax
#~~~~~~
fig = plt.figure(figsize=(5,5))
ax = fig.add_axes([0.12,0.12,0.5,0.4])
fontSize = 10
frameWidth = 0.5
xticks = np.arange(0,121,20)
xticklabels = [r'${}$'.format(x) for x in xticks]
#ax.set_xscale('log')
yticks =  np.linspace(0,30,num=6)
yticklabels = [r'${:.1f}$'.format(y) for y in yticks]
ax.set_xlim(xticks[0],xticks[-1])
ax.set_ylim(yticks[0],yticks[-1])
ax.set_xticks(xticks)
ax.set_yticks(yticks)
ax.set_xticklabels(xticklabels)
ax.set_yticklabels(yticklabels)
ax.set_xlabel(r'$t/t_0$',fontsize=10,labelpad=4)
ax.set_ylabel(r'$|\bm{F}_e|/F_s$',fontsize=10,labelpad=4)
ax.xaxis.set_tick_params(which='major',length=4,direction='in',width=frameWidth,pad=5)
ax.yaxis.set_tick_params(which='major',length=4,direction='in',width=frameWidth,pad=5)
ax.xaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
ax.yaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
ax.xaxis.set_ticks_position('both')
ax.yaxis.set_ticks_position('both')
for axis in ['top','bottom','left','right']:
	ax.spines[axis].set_linewidth(frameWidth)
for item in ([ax.title, ax.xaxis.label, ax.yaxis.label] + ax.get_xticklabels() + ax.get_yticklabels()):
	item.set_fontsize(fontSize)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Plot force against time
#~~~~~~~~~~~~~~~~~~~~~~~~~~~
my_path = '/home/gl5972/Documents/microorigami/scripts'
sides = args.sides
colors = ['#c6dbef','#9ecae1','#6baed6','#3182bd','#08519c']
for i,side in enumerate(sides):
	xs_force,ys_force = GetForce(my_path=my_path,ca=args.ca,side=side,ec=args.ec,every=50)
	xs_force = xs_force
	ax.plot(xs_force,
	        ys_force,
	        "-",
	        color = colors[::-1][i],
	        label = r'$N = {}$'.format(side))
ax.legend()
handles, labels = ax.get_legend_handles_labels()
ax.legend(handles,
	      labels,
	      loc='lower right',
	      frameon = True,
	      framealpha=1,
	      handletextpad = 0.2,
	      handlelength = 0.5)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Read side triangle without transform
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
m,rest = GetTopSide0(my_path=my_path,ca=args.ca,side=sides[0],ec=args.ec,time=0,transform=False)
xs_side = m[:,1]
ys_side = m[:,0]
xs_side = np.hstack((xs_side,xs_side[0]))
ys_side = np.hstack((ys_side,ys_side[0]))

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Create new axes + plot triangle side
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
x0 = 128
y0 = 0
dx = 30
dy = 17
height = 14
x0_all = [x0   ,x0+dx,x0,x0+dx]
y0_all = [y0+dy,y0+dy,y0,y0   ]
axs = []
for x0,y0 in zip(x0_all,y0_all):
	bb = ax.get_position()
	x0,y0 = TransformAxToFig(ax,x0,y0)
	alpha = (xs_side.max()-xs_side.min())/(ys_side.max()-ys_side.min())
	_,Ly = TransformAxToFig(ax,-1,height)
	Ly = Ly-bb.y0
	Lx = alpha*Ly
	axs.append(fig.add_axes([x0,y0,Lx,Ly]))
	axs[-1].axis('off')
	axs[-1].plot(xs_side,ys_side,color='black',linewidth=0.5)
	axs[-1].set_xticks([])
	axs[-1].set_yticks([])

#~~~~~~~~~~~~~~~~~
#  Select points
#~~~~~~~~~~~~~~~~~
xs_force,ys_force = GetForce(my_path=my_path,ca=args.ca,side=sides[0],ec=args.ec,every=50)
xs_select = np.array([10,25,30,50],dtype=int)
indices = np.ravel([np.argwhere(xs_force==x_select) for x_select in xs_select])
print(indices)
ys_select = np.take(ys_force,indices)
ax.plot(xs_select,
	    ys_select,
	    marker = 'o',
	    markersize = 5,
	    markerfacecolor = 'None',
	    markeredgecolor = 'C1',
	    markeredgewidth = 1,
	    linewidth = 0)
labels=['a','b','c','d']
for label,x,y in zip (labels,xs_select,ys_select):
	ax.text(x,y+0.5,r'${}$'.format(label),ha='center',va='bottom',color='C1')

#~~~~~~~~~~~~~~~~~~
#  Plot polylines
#~~~~~~~~~~~~~~~~~~
times = xs_select
print(times)
for i,(axi,time) in enumerate(zip(axs,times)):
	#Side transform properties
	m,rest = GetTopSide0(my_path=my_path,ca=args.ca,side=sides[0],ec=args.ec,time=time,transform=True)
	side_h = rest[0]
	side_n = rest[1]
	side_c = rest[2]
	side_q = rest[3]
	R = (side_q.inverse).rotation_matrix
	#m = np.array([R.dot(row-side_c) for row in m])
	tpcl_path = my_path + '/ca{}/side{}/ec{:.1f}'.format(args.ca,sides[0],args.ec)
	polylines = ImportPolylines(my_path=tpcl_path,name='tpcl',time=time,cloudsize=10,info=False)
	for polyline in polylines:
		if polyline.size>0:
			polyline = np.array([R.dot(row-side_c) for row in polyline])
			xs = polyline[:,1]
			ys = polyline[:,0]
			axi.plot(xs,ys,color='C1')
	force, torque = GetNormalizedCapillaryForce(my_path = tpcl_path,
		                                        name = 'tpcl',
		                                        theta = args.ca,
		                                        ec = args.ec,
		                                        time = time,
		                                        info = False)
	force_mag = np.linalg.norm(force)
	force_ref = GetStructuralReferenceForce(my_path=tpcl_path,name='hinge0',time=0)
	print("Force (c++/python) = {:.3f}\t{:.3f}".format(force_mag/force_ref,ys_select[i]))
	bb = axs[i].get_position()
	x0,y0 = bb.x0,bb.y0
	x1,y1 = bb.x1,bb.y1
	alpha = side_q.degrees
	ax.text((x0+x1)/2,y0,r'$\alpha = {:.0f}^\circ$'.format(alpha),ha='center',va='top',transform=fig.transFigure)
	ax.text((x0+x1)/2,y0+0.4*(y1-y0),r'${}$'.format(labels[i]),ha='center',va='center',color='C1',transform=fig.transFigure)

fig.savefig('plot.pdf')
plt.close(fig)
