#!/bin/bash

server="hemera"
owner="${HOME}/PaperLeipzig/Fig1"
copy_data="0"
export_data="0"
make_figure="0"
verbose="0"

function print_usage()
{
	echo "Copy latest time data from server and make Figure 1"
	echo "--help:        help"
	echo "--verbose:     verbose"
	echo "--server:      ssh server"
	echo "--server_dir:  directory on the server"
	echo "--copy:        copy data"
	echo "--export:      export to paraview"
	echo "--fig:         make figure"
	exit
}

function pass_args()
{
	local shorts='hvs:d:cef'
	local longs='help,verbose,server,server_dir:,copy,export,fig'
	local options=$(getopt --options ${shorts} --long ${longs} --name "$0" -- "$@")
	if [ $? != 0 ]; then echo "Failed parsing options." >&2 ; exit 1 ; fi
	eval set -- "${options}"
	while true; do
		case "$1" in
			-h | --help)       print_usage;     shift  ;;
			-v | --verbose)    verbose="1";     shift  ;;
			-s | --server)     server=$2;       shift 2;;
			-d | --server_dir) owner=$2;        shift 2;;
			-c | --copy)       copy_data="1";   shift  ;;
			-e | --export)     export_data="1"; shift  ;;
			-f | --fig)        make_figure="1"; shift  ;;
			--) shift ; break ;;
			* ) break ;;
		esac
	done

}

function Main()
{
	pass_args $0 "$@"
	local sidefolders=$(GetSideFolders ${server} ${owner})
	for sidefolder in ${sidefolders}
	do
		#sidefolders = owner + / + [side1,side2,...,sideX] with owner = directory and X=1,2,3,...,nsides
		local nsides=$(echo ${sidefolder} | awk -F '/' '{print $NF}' | sed s'/side//')
		local latest_time=$(GetLatestTime ${server} ${sidefolder})
		echo "n(side) = ${nsides}, latest time = ${latest_time}"
		if (( ${copy_data} ))
		then
			CopyFields ${server} ${sidefolder} ${latest_time}
		fi
		if (( ${export_data} ))
		then
			ExportFieldsToParaview ${nsides} ${latest_time}
			MakeParaviewImportScript ${nsides} ${latest_time}
		fi
	done
	if (( ${make_figure} )); then python3 fig1.py; fi
# 	ConcatenateParaviewMacros
}

GetSideFolders()
{
	local arg_server=${1}
	local arg_owner=${2}
	ssh ${arg_server} "ls -d ${arg_owner}/side[0-9]*"
}

GetLatestTime()
{
	local arg_server=${1}
	local arg_folder=${2}
	local cmd="ssh ${arg_server} \"ls ${arg_folder}/phi_t?????.dat | tail -n 1 | awk -F '/' '{print \\\\$NF}' | awk -vFS='(_t|.dat)' '{print \\\$2}'\"" #check NF
	local latest_time=$(eval ${cmd})
	echo $(expr ${latest_time} + 0)
}

CopyFields()
{
	local arg_server=${1}
	local arg_sidefolder=${2}
	local arg_time=$(printf "%05d" ${3})
	local base_path=$(dirname "$(realpath $0)")
	local nsides=$(echo ${arg_sidefolder} | awk -F '/' '{print $NF}' | sed s'/side//')
	mkdir -p ${base_path}/side${nsides}
	for (( i=0; i<${nsides}; i++ ))
	do
		scp ${arg_server}:${arg_sidefolder}/side${i}_t${arg_time}.dat* ${base_path}/side${nsides} &> /dev/null
	done
	scp ${arg_server}:${arg_sidefolder}/phi_t${arg_time}.dat* ${base_path}/side${nsides} &> /dev/null
	scp ${arg_server}:${arg_sidefolder}/base.dat* ${base_path}/side${nsides} &> /dev/null
	scp ${arg_server}:${arg_sidefolder}/Input.in ${base_path}/side${nsides} &> /dev/null
}

ExportFieldsToParaview()
{
	local arg_nsides=${1}
	local arg_time=$(printf "%05d" ${2})
	local base_path=$(dirname "$(realpath $0)")
	cd ${base_path}/side${arg_nsides}
	ExportFieldToVTS -f phi -t ${arg_time} #&> /dev/null
	ExportSolid -s base -n ${arg_nsides} -stl #&> /dev/null
	for (( i=0; i<${arg_nsides}; i++ ))
	do
		ExportSolid -s side${i} -n 3 -stl -t ${arg_time} #&> /dev/null
	done
	cd ${base_path}
}

MakeParaviewImportScript()
{
	local arg_nsides=${1}
	local arg_time=$(printf "%05d" ${2})
	local base_path=$(dirname "$(realpath $0)")
	local python_path="${base_path}/side${arg_nsides}"
	local python_script="${python_path}/pvimport.py"
	>${python_script}
	echo "from paraview.simple import *" >> ${python_script}
	echo "base_stl = STLReader(registrationName='base.stl', FileNames=['${python_path}/base.stl'])" >> ${python_script}
	for (( i=0; i<${arg_nsides}; i++ ))
	do
		echo "side${i}_stl = STLReader(registrationName='side${i}.stl', FileNames=['${python_path}/side${i}_t${arg_time}.stl'])" >> ${python_script}
	done
	echo "phi_vts = XMLStructuredGridReader(registrationName='phi.vts', FileName=['${python_path}/phi_t${arg_time}.vts'])" >> ${python_script}
	echo "phi_vts.PointArrayStatus = ['phi']" >> ${python_script}
	#echo "phi_ctr = Contour(registrationName='phi.ctr', Input=phi_vts)" >> ${python_script}
	#echo "phi_ctr.ContourBy = ['POINTS', 'phi']" >> ${python_script}
	#echo "phi_ctr.Isosurfaces = [0.5]" >> ${python_script}
	#echo "phi_ctr.PointMergeMethod = 'Uniform Binning'" >> ${python_script}
}

# ConcatenateParaviewMacros()
# {
# 	local base_path=$(dirname "$(realpath $0)")
# 	local sidefolders=$(ls -d ${base_path}/side[0-9]*)
# 	>importAll.py
# 	for sidefolder in ${sidefolders}
# 	do
# 		cat ${sidefolder}/pvimport.py >> importAll.py
# 	done
# }

Main "$@"
