#!/usr/bin/python3
import numpy as np
from Utils import *
import argparse
from pyquaternion import Quaternion
import matplotlib.pyplot as plt
from matplotlib.patches import Circle
from matplotlib.collections import PatchCollection
import matplotlib
matplotlib.rc('text', usetex=True)
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]

def GetFiber(fname,time):
	fname = fname + '_t{time:05}.txt'.format(time=time)
	#print("read",fname)
	r,x,y,z,qx,qy,qz,qw = np.loadtxt(fname,usecols=(1,4,5,6,7,8,9,10),unpack=True)
	length = (r.size-1) * (2*r[0])
	rs = r / length
	pts = np.stack([x,y,z],axis=-1)
	pts = (pts - pts[0])/ length
	qs = np.stack((qw,qx,qy,qz),axis=-1)
	qs = [Quaternion(q) for q in qs]
	return rs[0], pts, qs

def GetFiberTrajectories(fname,times):
	positions = [GetFiber(f,time)[1] for time in times]
	x = np.transpose(positions)[0]
	y = np.transpose(positions)[1]
	z = np.transpose(positions)[2]
	return x,y,z

#~~~~~~~~
#  Load
#~~~~~~~~
parser = argparse.ArgumentParser()
parser.add_argument('-loadn',type=int,default=31,help='')
parser.add_argument('-loadp',type=int,default=1,help='')
parser.add_argument('-t',type=StrToNumpy,default='0:20000:100',help='time')
args = parser.parse_args()
loadn = args.loadn
load_pic = args.loadp
times=args.t

#~~~~~~~~~~~~~~~~~
#  Plot A: fibre
#~~~~~~~~~~~~~~~~~
fig, ax = plt.subplots()
ax.set_aspect('equal')
#times = np.arange(0,20000,100)
f="/home/gl5972/Documents/microorigami/src/hinge0_load"+str(loadn-1)
#f="/home/gl5972/Documents/microorigami/src/hinge0_load"+str(1)
r,xss,qs = GetFiber(f,times[-1])
patches = []
for (xs,qi) in zip(xss,qs):
	c = Circle((xs[0],xs[2]),r)
	A = xs
	B = A + qi.rotate(np.array([r,0,0]))
	line = np.stack((A,B))
	ax.plot(line[:,0],line[:,2],'-',color='black')
	patches.append(c)
p = PatchCollection(patches)
ax.add_collection(p)
xss,yss,zss = GetFiberTrajectories(f,times)
errors = np.diff(zss[-1])
errors = -(errors - errors.min())/(errors.max()-errors.min())
iters = np.linspace(start=0,stop=1,num=errors.size,endpoint=True)
ax.plot(iters,errors,color='black')
xss,yss,zss = GetFiberTrajectories(f,times)
for (xs,zs) in zip(xss,zss):
	ax.plot(xs,zs,'.',color='black',ms=0.5)
fig.savefig('plota.pdf')
plt.close(fig)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  PlotB: deformation against load
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
fig = plt.figure(figsize=(5,5))
ax = fig.add_axes([0.1,0.1,0.8,0.6])
fontSize = 10
frameWidth = 0.5
xticks = np.arange(0,9)
xticklabels = [r'${:d}$'.format(int(x)) for x in xticks]
yticks = np.linspace(0,1,5)
ax.set_xlim(xticks[0],xticks[-1])
ax.set_ylim(yticks[0],yticks[-1])
ax.set_xticks(xticks)
ax.set_yticks(yticks)
ax.set_xticklabels(xticklabels)
ax.set_xlabel(r'$F_e/F_s$',fontsize=10,labelpad=3)
ax.set_ylabel(r'$\delta / \ell$',fontsize=10,labelpad=3)
ax.xaxis.set_tick_params(which='major',length=4,direction='in',width=frameWidth,pad=5)
ax.yaxis.set_tick_params(which='major',length=4,direction='in',width=frameWidth,pad=5)
ax.xaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
ax.yaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
ax.xaxis.set_ticks_position('both')
ax.yaxis.set_ticks_position('both')
for axis in ['top','bottom','left','right']:
	ax.spines[axis].set_linewidth(frameWidth)
for item in ([ax.title, ax.xaxis.label, ax.yaxis.label] + ax.get_xticklabels() + ax.get_yticklabels()):
	item.set_fontsize(fontSize)

#~~~~~~~~~
# Theory 1
#~~~~~~~~~
ax.plot([0,3],
        [0,1],
        '-',
        linewidth = 1,
        linestyle='dashed',
        color='C1',
        label='Small deformation theory')

#~~~~~~~~~
# Theory 2
#~~~~~~~~~
xth = [0.0,0.5,1.0,1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0,5.5,6.0,6.5,7.0,7.5,8.0,8.5,9.0,9.5,10.0]
yth = [0.0,0.16214,0.30172,0.41098,0.49346,0.55566,0.60325,0.64039,0.66996,0.69397,0.71379,0.73042,0.74457,0.75676,0.76737,0.77670,0.78498,0.79239,0.79906,0.80510,0.81061]
ax.plot(xth,
        yth,
        '-',
        linewidth = 1,
        linestyle='-',
        color='C1',
        label='Elastica theory (Fertis, 1999)')

#~~~~~~~~~~~~
# Simulations
#~~~~~~~~~~~~
loads = np.linspace(0,8,num=loadn,endpoint=True)
deltas = np.copy(loads)
for i,loadi in enumerate(loads):
	f='hinge0_load' + str(i)
	lastTime = times[-1]
	print("  read",f,"at time t =",lastTime)
	_,xss,_= GetFiber(f,lastTime)
	deltai = xss[-1][-1]
	deltas[i] = deltai
deltas = np.absolute(deltas)
ax.plot(loads,
        deltas,
        marker = 'o',
        markersize = 5,
        markerfacecolor = 'None',
        markeredgecolor = 'C0',
        markeredgewidth = 1,
        linewidth = 0,
        label = 'Present simulation')

#~~~~~~~~~
# Snapshot
#~~~~~~~~~
img='/home/gl5972/Documents/microorigami/scripts/fig5_hinge.png'
pic = plt.imread(img);
Nx,Ny,_ = pic.shape
x0,y0 = 4.8,0.04
Lx = 0.3
Ly = Ny/Nx*Lx
x0,y0 = TransformAxToFig(ax,x0,y0)
ax_img = fig.add_axes([x0,y0,Lx,Ly])
#ax_img.set_xlim(0,Nx)
#ax_img.set_ylim(0,Ny)
ax_img.set_xticks([])
ax_img.set_yticks([])
ax_img.imshow(pic)
for pos in ['right', 'top', 'bottom', 'left']:
    ax_img.spines[pos].set_visible(False)

#~~~~~~
# Arrow
#~~~~~~
loadi = load_pic
A = [loads[loadi],deltas[loadi]]
bbox_img = ax_img.get_position()
xb = 0.8*bbox_img.x0 + 0.2*bbox_img.x1
yb = bbox_img.y1
xb,yb = TransformFigToAx(ax,xb,yb)
B = [xb,yb]
Arrow(ax,
      A,
      B,
      zorder = 10,
      arrowstyle = '<|-',
      head_length = 0.4,
      head_width = 0.2,
      shrinkA = 5,
      shrinkB = 3,
      linewidth = 0.5,
      color = 'C0')

#~~~~~~~
# Legend
#~~~~~~~
handles, labels = ax.get_legend_handles_labels()
#sort=[1,2,0]
#labels = [labels[i] for i in sort]
#handles = [handles[i] for i in sort]
ax.legend(handles,
	      labels,
	      loc='upper left',
	      frameon = False)

fig.savefig('plotb.pdf',dpi=400)
