#!/bin/python3
import numpy as np
from scipy.optimize import fsolve
import matplotlib.pyplot as plt
from pathlib import Path
from itertools import zip_longest
from Utils import *
import matplotlib
import argparse
matplotlib.rc('text',usetex=True)
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}\usepackage{bm}"]

def GetSimulationData(folder='.',nxs=[100,150,200],thetas=np.arange(start=20,stop=141,step=10),max_it=-1):
	vs = [] #=volume[nx][theta][iteration]
	fs = [] #=force [nx][theta][iteration]
	for nx in nxs:
		for theta in thetas:
			my_file = folder + '/nx{nx:03d}/theta{theta:03d}/force.txt'.format(nx=nx,theta=theta)
			my_file = Path(my_file)
			if my_file.is_file():
				volume,force = np.loadtxt(my_file,delimiter=' ',usecols=(4,3),unpack=True)
				#if max_it>-1:
					#every = int(volume.size/max_it)
					#volume = volume[::every]
					#force = force[::every]
				vs.append(volume)
				fs.append(force)
			else:
				vs.append([np.nan]),
				fs.append([np.nan])
	vs = np.transpose(list(zip_longest(*vs,fillvalue=-1)))
	fs = np.transpose(list(zip_longest(*fs ,fillvalue=-1)))
	n0,n1,n2 = len(nxs),len(thetas),vs[0].size
	vs,fs = np.reshape(vs,(n0,n1,n2)),np.reshape(fs,(n0,n1,n2))
	#for i,j in np.ndindex((n0,n1)):
		#f,v = np.copy(fs[i][j]),np.copy(vs[i][j])
		#if f[-1] < 0:
			##Dummy correction for unfinished simulations
			#indices = np.argwhere(f<0).flatten()
			#index = indices[0]-1
			#fs[i][j][indices] = f[index]
			#vs[i][j][indices] = v[index]
	return vs,fs

def ReferenceForce(volume=1,sigma=1):
	pi = np.pi
	reference_length = np.cbrt(3*volume/(2*pi))
	reference_force = 2*pi*sigma*reference_length
	return reference_force

def GetTheoreticalForce(volume=1,sigma=1,small=0.001,num=100):
	pi = np.pi
	def GetCap(volume,angle):
		#r: sphere radius
		#h: cap height
		#a: cap base radius
		def f0(r,h):
			return (pi/3)*(h**2)*(3*r-h)
		def f1(a,h):
			return (pi/6)*(h)*(3*(a**2)+(h**2))
		def f2(r,angle):
			c = np.cos(angle)
			return (pi/3)*(r**3)*(2+c)*((1-c)**2)
		estimate = np.cbrt(volume)
		r = fsolve(lambda r:f2(r,angle)-volume,estimate)[0]
		h = fsolve(lambda h:f0(r,h    )-volume,estimate)[0]
		a = fsolve(lambda a:f1(a,h    )-volume,estimate)[0]
		return r,h,a
	def GetCapContour(volume,angle,printCap=False):
		r,h,a = GetCap(volume,angle)
		if printCap:
			print("r =",r)
			print("h =",h)
			print("a =",a)
			print("y0-(r-h) =",8-(r-h))
		t = np.linspace(0,2*pi,1000)
		x = r*np.cos(t)
		y = r*np.sin(t)-(r-h)
		return x[y>0],y[y>0],r,h,a
	thetas = np.linspace(0+small,pi-small,num,endpoint=True)
	forces = np.empty(thetas.shape)
	for i,thetai in enumerate(thetas):
		x,y,_,_,a = GetCapContour(volume,thetai)
		forces[i] = 2*pi*a
	thetas = np.concatenate(([0],thetas,[pi]))
	forces = np.concatenate(([0],forces,[0] ))
	forces = np.multiply(forces,np.sin(thetas))
	reference_force = ReferenceForce(volume,sigma)
	return thetas,forces/reference_force

##~~~~~~~~~~~~~~~~~~
## Check convergence
##~~~~~~~~~~~~~~~~~~
#fig, ax = plt.subplots()
#thetas = [20,40,60,80,90,100,120,140]
#volumes,forces = GetSimulationData(nxs=[150],thetas=thetas,max_it=-1)
#xs = np.arange(0,forces.shape[-1])
#colors = ['C0','C1','C2']
#for i,(vi,fi) in enumerate(zip(volumes[0:1],forces[0:1])):
	#for j,(vj,fj) in enumerate(zip(vi,fi)):
		#ys = fj
		#ax.text(xs[-1],ys[-1],'theta = ' + str(thetas[j]),ha='right',va='bottom')
		#label=''
		#if j==0:
			#label='nx = {}'.format('150')
		#ax.plot(xs,ys,'--',color=colors[i],label=label)
#ax.legend()
#fig.savefig('fig5_check.pdf')
#plt.close(fig)

def SetAxis(ax):
	fontSize=12
	frameWidth=0.5
	xticks = np.linspace(0,180,5)
	xticklabels = [r'${:d}$'.format(int(x)) for x in xticks]
	yticks = np.linspace(0,1.2,7)
	ax.set_xlim(xticks[0],xticks[-1])
	ax.set_ylim(yticks[0],yticks[-1])
	ax.set_xticks(xticks)
	ax.set_yticks(yticks)
	ax.set_xticklabels(xticklabels)
	ax.set_xlabel(r'$\gamma (^\circ)$',fontsize=fontSize,labelpad=5)
	ax.set_ylabel(r'$|\bm{F}_e|/F_c$',fontsize=fontSize,labelpad=5)
	ax.xaxis.set_tick_params(which='major',length=4,direction='in',width=frameWidth,pad=5)
	ax.yaxis.set_tick_params(which='major',length=4,direction='in',width=frameWidth,pad=5)
	ax.xaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
	ax.yaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
	ax.xaxis.set_ticks_position('both')
	ax.yaxis.set_ticks_position('both')
	for axis in ['top','bottom','left','right']:
		ax.spines[axis].set_linewidth(frameWidth)
	for item in ([ax.title, ax.xaxis.label, ax.yaxis.label] + ax.get_xticklabels() + ax.get_yticklabels()):
		item.set_fontsize(fontSize)

#~~~~~~~~~~
# Load data
#~~~~~~~~~~
nxs = [100,150,200]
thetas = [20,40,60,80,90,100,120,140]
volumes,forces = GetSimulationData(nxs=nxs,thetas=thetas)

#~~~~~~~
# Print
#~~~~~~~
for i,nx in enumerate(nxs):
	for j,theta in enumerate(thetas):
		my_path = 'nx{nx:03d}/theta{theta:03d}'.format(nx=nx,theta=theta)
		polylines = ImportPolylines(my_path=my_path,name='tpcl00',time=100,info=False)
		length = GetPolylineLength(polylines[0])
		if length>0:
			s = np.sin(theta*np.pi/180)
			err = np.abs(forces[i][j][-1]-length*s)/(length*s)
			print("(nx,theta,err) = {} {} {:.3f}".format(nx,theta,err))

#~~~~~~~~~~~
# Set figure
#~~~~~~~~~~~
fig = plt.figure(figsize=(6,4)) #figsize=(10,10)
margin = 0.2
x0 = margin
y0 = margin
Lx = 1-2*margin
Ly = 1-2*margin
ax = fig.add_axes([x0,y0,Lx,Ly])
SetAxis(ax)
x_th,y_th = GetTheoreticalForce()
ax.plot(x_th*180/np.pi,
        y_th,
        linewidth = 2,
        color = 'C1',
        label = r'$\textrm{Theory}$')
markers = ['s','o','d']
colors = ['#08519c','#3182bd','#6baed6']
for i,(vi,fi) in enumerate(zip(volumes,forces)):
	fs = fi[:,-1]
	vs = vi[:,-1]
	sigma = 1
	fs_ref = np.array([ReferenceForce(v,sigma) for v in vs])
	fs = np.divide(fs,fs_ref)
	#fs = np.divide(fs,np.sin(thetas)*np.pi/180)
	ax.plot(thetas,
	        fs,
	        linewidth = 0,
	        marker = markers[i],
	        markersize = 4,
	        markerfacecolor = 'None',
	        markeredgecolor = colors[i],
	        markeredgewidth = 1,
	        label = r'$N = {:d}^3$'.format(nxs[i]))
handles, labels = ax.get_legend_handles_labels()
sort = [1,2,3,0]
labels = [labels[i] for i in sort]
handles = [handles[i] for i in sort]
ax.legend(handles,
	      labels,
	      loc='upper right',
	      frameon = True,
	      framealpha=1,
	      handletextpad = 0.2,
	      handlelength = 0.5)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Overlay simulation snapshots
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
thetas_img = [40,80,120]
imgs=['fig5_theta40.png','fig5_theta80.png','fig5_theta120.png']
ax_positions=[[0.16,0.95],[0.50,0.95],[0.84,0.95]]
ax_size = 0.2
for img,theta_img in enumerate(thetas_img):
	pic = plt.imread(imgs[img]);
	x = 114
	y = 275
	w = 1234
	h = 446
	pic = pic[y:y+h,x:x+w]
	bbox = ax.get_position()
	x0 = bbox.x0 + bbox.width  * (ax_positions[img][0]) - ax_size/2
	y0 = bbox.y0 + bbox.height * (ax_positions[img][1])
	Lx = ax_size
	Ly = ax_size
	ax_new = fig.add_axes([x0,y0,Lx,Ly])
	ax_new.axis('off')
	ax_new.imshow(pic)
	item = np.argwhere(np.array(thetas) == theta_img).flatten()[0]
	theta = thetas[item]
	volume = volumes[-1][item][-1]
	force = forces[-1][item][-1] / ReferenceForce(volume,1)
	#print(theta,force)
	bbox = ax_new.get_position()
	xa,ya = (bbox.x0+bbox.x1)/2,bbox.y0
	xa,ya = TransformFigToAx(ax,xa,ya)
	A = np.array([xa,ya])
	B = [theta,force]
	#print(A,B)
	Arrow(ax,A,B,arrowstyle='-|>',shrinkB=10,color='C0')

fig.savefig('fig5.pdf',dpi=400)

#plt.close()


