#!/bin/bash

server="hemera"
owner="${HOME}/Leipzig/Fig5"
copy_data="0"
export_data="0"
make_figure="0"
clean_data="0"
verbose="0"

function print_usage()
{
	echo "Copy latest time data from server and make figure 5"
	echo "--help:       help"
	echo "--verbose:    verbose"
	echo "--server:     ssh server"
	echo "--server_dir: directory on the server"
	echo "--copy:       copy data"
	echo "--export:     export to paraview"
	echo "--fig:        make figure"
	exit
}

function pass_args()
{
	local shorts='hvs:d:cef'
	local longs='help,verbose,server,server_dir:,copy,export,fig'
	local options=$(getopt --options ${shorts} --long ${longs} --name "$0" -- "$@")
	if [ $? != 0 ]; then echo "Failed parsing options." >&2 ; exit 1 ; fi
	eval set -- "${options}"
	while true; do
		case "$1" in
			-h | --help)       print_usage;     shift  ;;
			-v | --verbose)    verbose="1";     shift  ;;
			-s | --server)     server=$2;       shift 2;;
			-d | --server_dir) owner=$2;        shift 2;;
			-c | --copy)       copy_data="1";   shift  ;;
			-e | --export)     export_data="1"; shift  ;;
			-f | --fig)        make_figure="1"; shift  ;;
			--) shift ; break ;;
			* ) break ;;
		esac
	done

}

function Main()
{
	pass_args $0 "$@"
	local nxfolders=$(GetNxFolders ${server} ${owner}) #nxfolders = owner + / + [nx100,nx200,...]
	for nxfolder in ${nxfolders}
	do
		local nx=$(echo ${nxfolder} | awk -F '/' '{print $NF}' | sed s'/nx//')
		local thetas=$(GetThetas ${server} ${nxfolder})
		for theta in ${thetas}
		do
			if (( ${copy_data} )); then rm -f ${nxfolder}; CopyFields ${server} ${owner} ${nx} ${theta}; fi
			if (( ${export_data} )); then ExportFieldsToParaview ${nx} ${theta}; fi
		done
	#echo "${thetas[*]}"
	done
	if (( ${make_figure} )); then python3 fig5.py; fi
}

function GetNxFolders()
{
	local arg_server=${1}
	local arg_owner=${2}
	ssh ${arg_server} "ls -d ${arg_owner}/nx[0-9]*"
}

function GetLatestTime()
{
	local arg_server=${1}
	local arg_folder=${2}
	ssh ${arg_server} "test -e ${arg_folder}/phi_t00000.dat"
	if [ $? -eq 0 ]
	then
		local latest_time=$(ssh ${arg_server} \
		"ls ${arg_folder}/phi_t?????.dat | tail -n 1 | awk -F '/' '{print \$NF}' | awk -vFS='(_t|.dat)' '{print \$2}'")
		echo $(expr ${latest_time} + 0)
	else
		echo "-1"
	fi
}

function GetThetas()
{
	local arg_server=${1}
	local arg_folder=${2}
	local thetas=($(ssh ${arg_server} "ls -d ${arg_folder}/theta[0-9][0-9][0-9]"))
	for i in "${!thetas[@]}" #bash array
	do
		local theta=$(echo ${thetas[${i}]} | awk -F '/' '{print $NF}' | sed 's/theta//')
		thetas[${i}]=$(echo $(expr ${theta} + 0))
	done
	echo ${thetas[@]}
}

function CopyFields()
{
	local arg_server=${1}
	local arg_owner=${2}
	local arg_nx=${3}
	local arg_theta=$(printf "%03d" ${4})
	local base_path=$(dirname "$(realpath $0)")
	local server_dir="${arg_owner}/nx${arg_nx}/theta${arg_theta}"
	local new_dir="${base_path}/nx${arg_nx}/theta${arg_theta}"
	local latest_time=$(GetLatestTime ${arg_server} ${server_dir})
	local latest_time05=$(printf "%05d" ${latest_time})
	echo "copy nx = ${arg_nx} theta = $(expr ${arg_theta} + 0) t_max = ${latest_time}"
	if (( ${latest_time} < 0 ))
	then
		echo "  empty directory"
	else
		mkdir -p ${new_dir}
		scp ${arg_server}:${server_dir}/force.txt ${new_dir} &> /dev/null
		scp ${arg_server}:${server_dir}/Input.in ${new_dir} &> /dev/null
		scp ${arg_server}:${server_dir}/log_output ${new_dir} &> /dev/null
		scp ${arg_server}:${server_dir}/log_error ${new_dir} &> /dev/null
		scp ${arg_server}:${server_dir}/base.dat* ${new_dir} &> /dev/null
		scp ${arg_server}:${server_dir}/phi_t${latest_time05}.dat* ${new_dir} &> /dev/null
		scp ${arg_server}:${server_dir}/tpcl_t${latest_time05}.dat* ${new_dir} &> /dev/null
	fi
}

function ExportFieldsToParaview()
{
	local arg_nx=${1}
	local arg_theta=$(printf "%03d" ${2})
	local base_path=$(dirname "$(realpath $0)")
	local new_dir="${base_path}/nx${arg_nx}/theta${arg_theta}"
	echo "export nx = ${arg_nx} theta = $(expr ${arg_theta} + 0)"
	if [ -d "${new_dir}" ]
	then
		var=$(find ${new_dir} -maxdepth 1 -name 'phi_t?????.dat' -printf '1' -quit)
		if [ -z "${var}" ]
		then
			echo "  empty directory"
		else
			local latest_time=$(expr 0 + $(ls ${new_dir}/phi_t?????.dat | tail -n 1 | awk -F '/' '{print $NF}' | awk -vFS='(_t|.dat)' '{print $2}'))
			local latest_time05=$(printf "%05d" ${latest_time})
			cd ${new_dir}
			[ -f "base.dat" ] && ExportSolid -f base -n 4 -stl &> /dev/null
			[ -f "phi_t${latest_time05}.dat" ] && ExportFieldToVTS -f phi -t ${latest_time} &> /dev/null
			[ -f "tpcl_t${latest_time05}.dat" ] && ExportPolylines -f tpcl -t ${latest_time} &> /dev/null
			[ -f "tpcl00_t${latest_time05}.txt" ] && PolylineToVTK.py -f tpcl00 -t ${latest_time} &> /dev/null
			[ -f "tpcl00_t${latest_time05}.py" ] && pvpython tpcl00_t${latest_time05}.py &> /dev/null
			cd ${base_path}
		fi
	else
		echo "  directory does not exist"
	fi
}

Main "$@"
