from Utils import *
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import argparse
from pyquaternion import Quaternion
from scipy.optimize import curve_fit
matplotlib.rc('text', usetex=True)
matplotlib.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]

def GetSide(fname,time):
	if time<0:
		fname = fname + '.txt'
	else:
		fname = fname + '_t{t:05d}.txt'.format(t=time)
	data = np.loadtxt(fname,usecols=0,unpack=True)
	m = np.reshape(data[0:9],(3,3)).transpose()
	h = data[9]
	n = data[10:13]
	c = data[13:16]
	q = Quaternion(data[19],data[16],data[17],data[18])
	m = np.array([c + q.rotate(row) for row in m])
	n = q.rotate(n)
	return m,h,n,c,q

def GetFiber(fname,time):
	fname = fname + '_t{time:05}.txt'.format(time=time)
	r,x,y,z,qx,qy,qz,qw = np.loadtxt(fname,usecols=(1,4,5,6,7,8,9,10),unpack=True)
	length = (r.size-1) * (2*r[0])
	rs = r / length
	pts = np.stack([x,y,z],axis=-1)
	pts = (pts - pts[0])/ length
	qs = np.stack((qw,qx,qy,qz),axis=-1)
	qs = [Quaternion(q) for q in qs]
	return rs[0], pts, qs

def ParaviewScript(my_path='.',ca=90,side=3,ec=0.1,t=400):
	d='{}/ca{}/side{}/ec{:.1f}'.format(my_path,ca,side,ec)
	print('from paraview.simple import *')
	print('base = STLReader(registrationName = \'base\', FileNames = [\'{}/base.stl\'])'.format(d))
	for s in range(side):
		print('side{s} = STLReader(registrationName = \'side{s}\', FileNames = [\'{d}/side{s}_t{t:05}.stl\'])'.format(s=s,d=d,t=t))
	print('drop = LegacyVTKReader(registrationName=\'drop\', FileNames=[\'{}/drop_t{:05}.vtk\'])'.format(d,t))
	print('renderView = GetActiveViewOrCreate(\'RenderView\')')
	print('renderView.OrientationAxesVisibility = 0')
	print('base_display = Show(base, renderView, \'GeometryRepresentation\')')
	print('ColorBy(base_display, None)')
	for s in range(side):
		print('side{s:}_display = Show(side{s:}, renderView, \'GeometryRepresentation\')'.format(s=s))
		print('ColorBy(side{}_display, None)'.format(s))
	print('drop_display = Show(drop, renderView, \'GeometryRepresentation\')')
	print('ColorBy(drop_display, None)')
	print('drop_display.AmbientColor = [0.0, 0.666, 1.0]')
	print('drop_display.DiffuseColor = [0.0, 0.666, 1.0]')
	print('drop_display.Opacity = 0.8')
	return 0

#~~~~~~~~~~
#  Parser
#~~~~~~~~~~
parser = argparse.ArgumentParser()
parser.add_argument('-t',type=StrToNumpy,default='0:400:20',help='time')
parser.add_argument('-ca',type=int,default=90,help='ca')
args = parser.parse_args()

#~~~~~~~~~~
#  Arrays
#~~~~~~~~~~
times = args.t
ca = args.ca
ecs = np.array([0.2,0.6,1.0,1.6])
folders = ["ca{}/side4/ec{:.1f}".format(ca,ec) for ec in ecs]
alphass = np.empty((len(folders),times.size))
for fi,folder in enumerate(folders):
	for ti,time in enumerate(times):
		_,_,_,_,q = GetSide(folder + "/side0", time)
		alphass[fi][ti] = q.degrees

for ec in ecs:
	ParaviewScript(ca=ca,side=5,ec=ec)

#~~~~~~~~~
# new axe
#~~~~~~~~~
fig = plt.figure(figsize=(5,5))
ax = fig.add_axes([0.1,0.1,0.6,0.4])
fontSize = 10
frameWidth = 0.5
xticks = np.linspace(0,300,num=7)
xticklabels = [r'${:d}$'.format(int(x)) for x in xticks]
yticks =  np.arange(0,121,30)
yticklabels = [r'${:d}^\circ$'.format(int(y)) for y in yticks]
ax.set_xlim(xticks[0],xticks[-1])
ax.set_ylim(yticks[0],yticks[-1])
ax.set_xticks(xticks)
ax.set_yticks(yticks)
ax.set_xticklabels(xticklabels)
ax.set_yticklabels(yticklabels)
ax.set_xlabel(r'$t/(t_0)$',fontsize=10,labelpad=4)
ax.set_ylabel(r'$\alpha$',fontsize=10,labelpad=2)
ax.xaxis.set_tick_params(which='major',length=4,direction='in',width=frameWidth,pad=5)
ax.yaxis.set_tick_params(which='major',length=4,direction='in',width=frameWidth,pad=5)
ax.xaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
ax.yaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
ax.xaxis.set_ticks_position('both')
ax.yaxis.set_ticks_position('both')
for axis in ['top','bottom','left','right']:
	ax.spines[axis].set_linewidth(frameWidth)
for item in ([ax.title, ax.xaxis.label, ax.yaxis.label] + ax.get_xticklabels() + ax.get_yticklabels()):
	item.set_fontsize(fontSize)

#~~~~~~~~~~~
# Plot data
#~~~~~~~~~~~
#https://colorbrewer2.o
colors=['#bdd7e7','#6baed6','#3182bd','#08519c']
for i,alphas in enumerate(alphass):
	ax.plot(times,
	        alphas,
	        color =colors[i],
	        label=r'$Ec = {}$'.format(ecs[i]))

#~~~~~~~~~~~~~
# Plot images
#~~~~~~~~~~~~~
img0 = plt.imread("fig8_s4_ec0.1.png")
img1 = plt.imread("fig8_s4_ec0.5.png")
img2 = plt.imread("fig8_s4_ec1.0.png")
img3 = plt.imread("fig8_s4_ec1.6.png")
for img,alphas in zip([img0,img1,img2,img3],alphass):
	L = 0.15
	bb = ax.get_position()
	x = 220
	y = alphas[-1] + 12
	x,y = TransformAxToFig(ax,x,y)
	ax_tmp = fig.add_axes([x,y-L/2,L,L])
	ax_tmp.imshow(img)
	ax_tmp.axis('off')

#~~~~~~~~
# Legend
#~~~~~~~~
ax.legend()
handles, labels = ax.get_legend_handles_labels()
sort=[3,2,1,0]
labels = [labels[i] for i in sort]
handles = [handles[i] for i in sort]
ax.legend(handles,
	      labels,
	      loc='upper left',
	      frameon = False,
	      handletextpad = 0.2,
	      handlelength = 0.5)

fig.savefig('plot.pdf',dpi=400)
plt.close(fig)
